/*
 * File:   main.c
 * Author: toffi
 *
 * Created on May 3, 2016, 4:13 PM
 */


#include "xc.h"
#include "pic-config.h"

// TODO: get rid of this, it's for the disgusting software delays!
#define FOSC 7372800
#define FCY (FOSC/2)
#include  <libpic30.h>
int i;


void setVoltage(int voltage); // function to set the motor voltage via PWM

int main(void)
{
    // ========================================================================
    // Step 0: Oscillator configuration for main clock
    
    // ========================================================================
    // Step 1: ACLK configuration for PWM
    // The PWM Module needs a 112-120 MHz ACLK to operate (why?)
    ACLKCONbits.FRCSEL = 1; // select FRC (7.37 MHz) as clock source for ACLK
    ACLKCONbits.SELACLK = 1; // switch on auxiliary oscillator
    ACLKCONbits.APSTSCLR = 7; // Clock Divider = 1
    ACLKCONbits.ENAPLL = 1; // enable auxiliary PLL for ACLK = 7.37MHz * 16
    while(ACLKCONbits.APLLCK != 1); // wait for PLL lock
    // If we made it here, we have ACLK running at ~118MHz
    
    // ========================================================================
    // Step 2: Basic PWM configuration
    // The motor bridge is connected to PWM1H/L ("positive" motor terminal) and
    // PWM2H/L ("negative" motor terminal), so we need those two.
    // At the moment, I use the prescaler to get down to a PWM frequency
    // of ~0.8kHz to be safe from any high frequency switching problems
    PTCON2bits.PCLKDIV = 5; // Divide-by-32 from ~29.3kHz
    // I work with a resolution of 15 bits here
    PTPER = 0x7FFF;
    // The discrete H-Bridge has no automatic dead-time generation, so dead
    // time has to be done by the PWM module
    // setting dead time to roughly 100ns as a first try
    DTR1 = 100;
    ALTDTR1 = 100;
    DTR2 = 100;
    ALTDTR2 = 100;
    // We are using four pins (PWM1L to PWM2H), so we need to configure them:
    __builtin_write_PWMSFR(&IOCON1, 0xC000, &PWMKEY);
    __builtin_write_PWMSFR(&IOCON2, 0xC000, &PWMKEY);
    // After a reset, the input for external overcurrent shutoff is switched on
    // I can't imagine the kind of moron you have to be to design it like that,
    // but you have to switch it off before using the PWM module:
    __builtin_write_PWMSFR(&FCLCON1, 0x0003, &PWMKEY);
    __builtin_write_PWMSFR(&FCLCON2, 0x0003, &PWMKEY);
    // The last step is to activate the PWM module:
    PTCONbits.PTEN = 1;
    // PWM should be active now
    // Setting a duty cycle should be as simple as writing to PDC1 / PDC2
    // TODO: Trigger ADC current measurement / BEMF measurement from PWM
    // TODO: Optimize PWM dead time for H-Bridge
    
    // ========================================================================
    // Step 3: UART configuration for RS485
    
    // ========================================================================
    // Step 4: ADC configuration for motor current- / BEMF-measurement
    
    // ========================================================================
    // Step 5: System timer configuration
    
    // ========================================================================
    // Step 6: Interrupt configuration
    
    // Main loop: non-timed tasks
    while(1)
    {
        for(i=0;i<20000;i++)
        {
            setVoltage(i);
            __delay_us(100);
        }
        for(i=19999;i>=0;i--)
        {
            setVoltage(i);
            __delay_us(100);
        }
        __delay_ms(1000);
        for(i=0;i>-20000;i--)
        {
            setVoltage(i);
            __delay_us(100);
        }
        for(i=-19999;i<=0;i++)
        {
            setVoltage(i);
            __delay_us(100);
        }
        __delay_ms(1000);
    }
    return 0;
}

void setVoltage(int voltage)
{
    if(voltage < 0) // negative voltage
    {
        PDC1 = 0; // "positive" terminal gets permanent GND
        PDC2 = 0-voltage; // "negative" terminal gets PWM
    }
    else
    {
        PDC2 = 0; // "negative" terminal gets permanent GND
        PDC1 = voltage;
    }
}